@@ -19,7 +19,7 @@ import numbers
 import warnings
 
 import numpy as np
-from numpy import dot, eye, ndarray, ones_like, sqrt, take, transpose, triu
+from numpy import ndarray, ones_like, sqrt
 from numpy.dual import inv
 from numpy.linalg import LinAlgError
 from scipy.optimize import basinhopping as scipy_basinhopping
@@ -33,14 +33,14 @@ from scipy.version import version as scipy_version
 import six
 import uncertainties
 
-from .parameter import Parameter, Parameters
-
 from ._ampgo import ampgo
+from .parameter import Parameter, Parameters
 
 # check for EMCEE
 try:
-    import emcee as emcee
+    import emcee
     HAS_EMCEE = True
+    EMCEE_VERSION = int(emcee.__version__[0])
 except ImportError:
     HAS_EMCEE = False
 
@@ -53,6 +53,13 @@ except ImportError:
     HAS_PANDAS = False
     isnull = np.isnan
 
+# check for numdifftools
+try:
+    import numdifftools as ndt
+    HAS_NUMDIFFTOOLS = True
+except ImportError:
+    HAS_NUMDIFFTOOLS = False
+
 # define the namedtuple here so pickle will work with the MinimizerResult
 Candidate = namedtuple('Candidate', ['params', 'score'])
 
@@ -113,6 +120,7 @@ class MinimizerException(Exception):
 
 class AbortFitException(MinimizerException):
     """Raised when a fit is aborted by the user."""
+
     pass
 
 
@@ -131,10 +139,10 @@ SCALAR_METHODS = {'nelder': 'Nelder-Mead',
                   'differential_evolution': 'differential_evolution'}
 
 # FIXME: update this when incresing the minimum scipy version
-major, minor, micro = np.array(scipy_version.split('.'), dtype='int')
-if (major >= 1 and minor >= 1):
+major, minor, micro = scipy_version.split('.', 2)
+if (int(major) >= 1 and int(minor) >= 1):
     SCALAR_METHODS.update({'trust-constr': 'trust-constr'})
-if major >= 1:
+if int(major) >= 1:
     SCALAR_METHODS.update({'trust-exact': 'trust-exact',
                            'trust-krylov': 'trust-krylov'})
 
@@ -229,8 +237,8 @@ class MinimizerResult(object):
         useful for understanding the values in :attr:`init_vals` and
         :attr:`covar`.
     covar : numpy.ndarray
-        Covariance matrix from minimization (`leastsq` only), with
-        rows and columns corresponding to  :attr:`var_names`.
+        Covariance matrix from minimization, with rows and columns
+        corresponding to :attr:`var_names`.
     init_vals : list
         List of initial values for variable parameters using :attr:`var_names`.
     init_values : dict
@@ -283,7 +291,7 @@ class MinimizerResult(object):
 
     @property
     def flatchain(self):
-        """A flatchain view of the sampling chain from the `emcee` method."""
+        """Show flatchain view of the sampling chain from `emcee` method."""
         if hasattr(self, 'chain'):
             if HAS_PANDAS:
                 if len(self.chain.shape) == 4:
@@ -299,9 +307,9 @@ class MinimizerResult(object):
             return None
 
     def show_candidates(self, candidate_nmb='all'):
-        """A pretty_print() representation of the candidates.
+        """Show pretty_print() representation of candidates from `brute` method.
 
-        Showing candidates (default is 'all') or the specified candidate-#
+        Showing all stored candidates (default) or the specified candidate-#
         from the `brute` method.
 
         Parameters
@@ -311,20 +319,22 @@ class MinimizerResult(object):
 
         """
         if hasattr(self, 'candidates'):
-            try:
-                candidate = self.candidates[candidate_nmb]
+            if candidate_nmb == 'all':
+                for i, candidate in enumerate(self.candidates):
+                    print("\nCandidate #{}, chisqr = "
+                          "{:.3f}".format(i+1, candidate.score))
+                    candidate.params.pretty_print()
+            elif (candidate_nmb < 1 or candidate_nmb > len(self.candidates)):
+                raise ValueError("'candidate_nmb' should be between 1 and {}."
+                                 .format(len(self.candidates)))
+            else:
+                candidate = self.candidates[candidate_nmb-1]
                 print("\nCandidate #{}, chisqr = "
                       "{:.3f}".format(candidate_nmb, candidate.score))
                 candidate.params.pretty_print()
-            except IndexError:
-                for i, candidate in enumerate(self.candidates):
-                    print("\nCandidate #{}, chisqr = "
-                          "{:.3f}".format(i, candidate.score))
-                    candidate.params.pretty_print()
 
     def _calculate_statistics(self):
         """Calculate the fitting statistics."""
-
         self.nvarys = len(self.init_vals)
         if isinstance(self.residual, ndarray):
             self.chisqr = (self.residual**2).sum()
@@ -334,7 +344,7 @@ class MinimizerResult(object):
             self.chisqr = self.residual
             self.ndata = 1
             self.nfree = 1
-        self.redchi = self.chisqr / self.nfree
+        self.redchi = self.chisqr / max(1, self.nfree)
         # this is -2*loglikelihood
         _neg2_log_likel = self.ndata * np.log(self.chisqr / self.ndata)
         self.aic = _neg2_log_likel + 2 * self.nvarys
@@ -352,7 +362,7 @@ class Minimizer(object):
 
     def __init__(self, userfcn, params, fcn_args=None, fcn_kws=None,
                  iter_cb=None, scale_covar=True, nan_policy='raise',
-                 reduce_fcn=None, **kws):
+                 reduce_fcn=None, calc_covar=True, **kws):
         """
         Parameters
         ----------
@@ -379,8 +389,7 @@ class Minimizer(object):
             the iteration number, `resid` the current residual array, and `*fcn_args`
             and `**fcn_kws` are passed to the objective function.
         scale_covar : bool, optional
-            Whether to automatically scale the covariance matrix (default is True,
-            `leastsq` only).
+            Whether to automatically scale the covariance matrix (default is True).
         nan_policy : str, optional
             Specifies action if `userfcn` (or a Jacobian) returns NaN
             values. One of:
@@ -407,6 +416,10 @@ class Minimizer(object):
 
             - callable : must take one argument (`r`) and return a float.
 
+        calc_covar : bool, optional
+            Whether to calculate the covariance matrix (default is True) for
+            solvers other than `leastsq` and `least_squares`. Requires the
+            `numdifftools` package to be installed.
         **kws : dict, optional
             Options to pass to the minimizer being used.
 
@@ -439,6 +452,7 @@ class Minimizer(object):
             self.userkws = {}
         self.kws = kws
         self.iter_cb = iter_cb
+        self.calc_covar = calc_covar
         self.scale_covar = scale_covar
         self.nfev = 0
         self.nfree = 0
@@ -669,6 +683,114 @@ class Minimizer(object):
         """
         pass
 
+    def _calculate_covariance_matrix(self, fvars):
+        """Calculate the covariance matrix.
+
+        The `numdiftoools` package is used to estimate the Hessian matrix, and
+        the covariance matrix is calculated as:
+
+        .. math::
+
+            cov_x = inverse(Hessian) * 2.0
+
+        Parameters
+        ----------
+        fvars : numpy.ndarray
+            Array of the optimal internal, freely variable parameters.
+
+        Returns
+        -------
+        cov_x : numpy.ndarray or None
+            Covariance matrix if successful, otherwise None.
+
+        """
+        warnings.filterwarnings(action="ignore", module="scipy",
+                                message="^internal gelsd")
+
+        nfev = deepcopy(self.result.nfev)
+        try:
+            Hfun = ndt.Hessian(self.penalty)
+            hessian_ndt = Hfun(fvars)
+            cov_x = inv(hessian_ndt) * 2.0
+        except (LinAlgError, ValueError):
+            return None
+        finally:
+            self.result.nfev = nfev
+
+        return cov_x
+
+    def _int2ext_cov_x(self, cov_int, fvars):
+        """Transform covariance matrix to external parameter space.
+
+        It makes use of the gradient scaling according to the MINUIT recipe:
+
+            cov_ext = np.dot(grad.T, grad) * cov_int
+
+        Parameters
+        ----------
+        cov_int : numpy.ndarray
+            Covariance matrix in the internal parameter space.
+        fvars : numpy.ndarray
+            Array of the optimal internal, freely variable, parameter values.
+
+        Returns
+        -------
+        cov_ext : numpy.ndarray
+            Covariance matrix, transformed to external parameter space.
+
+        """
+        g = [self.result.params[name].scale_gradient(fvars[i]) for i, name in
+             enumerate(self.result.var_names)]
+        grad2d = np.atleast_2d(g)
+        grad = np.dot(grad2d.T, grad2d)
+
+        cov_ext = cov_int * grad
+        return cov_ext
+
+    def _calculate_uncertainties_correlations(self):
+        """Calculate parameter uncertainties and correlations."""
+        self.result.errorbars = True
+
+        if self.scale_covar:
+            self.result.covar *= self.result.redchi
+
+        vbest = np.atleast_1d([self.result.params[name].value for name in
+                               self.result.var_names])
+
+        has_expr = False
+        for par in self.result.params.values():
+            par.stderr, par.correl = 0, None
+            has_expr = has_expr or par.expr is not None
+
+        for ivar, name in enumerate(self.result.var_names):
+            par = self.result.params[name]
+            par.stderr = sqrt(self.result.covar[ivar, ivar])
+            par.correl = {}
+            try:
+                self.result.errorbars = self.result.errorbars and (par.stderr > 0.0)
+                for jvar, varn2 in enumerate(self.result.var_names):
+                    if jvar != ivar:
+                        par.correl[varn2] = (self.result.covar[ivar, jvar] /
+                                             (par.stderr * sqrt(self.result.covar[jvar, jvar])))
+            except ZeroDivisionError:
+                self.result.errorbars = False
+
+        if has_expr:
+            try:
+                uvars = uncertainties.correlated_values(vbest, self.result.covar)
+            except (LinAlgError, ValueError):
+                uvars = None
+
+            # for uncertainties on constrained parameters, use the calculated
+            # "correlated_values", evaluate the uncertainties on the constrained
+            # parameters and reset the Parameters to best-fit value
+            if uvars is not None:
+                for par in self.result.params.values():
+                    eval_stderr(par, uvars, self.result.var_names, self.result.params)
+                # restore nominal values
+                for v, nam in zip(uvars, self.result.var_names):
+                    self.result.params[nam].value = v.nominal_value
+
     def scalar_minimize(self, method='Nelder-Mead', params=None, **kws):
         """Scalar minimization using :scipydoc:`optimize.minimize`.
 
@@ -799,16 +921,22 @@ class Minimizer(object):
             result.x = np.atleast_1d(result.x)
             result.residual = self.__residual(result.x)
             result.nfev -= 1
-        else:
-            pass
 
         result._calculate_statistics()
 
+        # calculate the cov_x and estimate uncertanties/correlations
+        if (not result.aborted and self.calc_covar and HAS_NUMDIFFTOOLS and
+                len(result.residual) > len(result.var_names)):
+            _covar_ndt = self._calculate_covariance_matrix(result.x)
+            if _covar_ndt is not None:
+                result.covar = self._int2ext_cov_x(_covar_ndt, result.x)
+                self._calculate_uncertainties_correlations()
+
         return result
 
     def emcee(self, params=None, steps=1000, nwalkers=100, burn=0, thin=1,
               ntemps=1, pos=None, reuse_sampler=False, workers=1,
-              float_behavior='posterior', is_weighted=True, seed=None):
+              float_behavior='posterior', is_weighted=True, seed=None, progress=True):
         r"""Bayesian sampling of the posterior distribution using `emcee`.
 
         Bayesian sampling of the posterior distribution for the parameters
@@ -1128,8 +1256,12 @@ class Minimizer(object):
             self.sampler.random_state = rng.get_state()
 
         # now do a production run, sampling all the time
-        output = self.sampler.run_mcmc(p0, steps)
-        self._lastpos = output[0]
+        if EMCEE_VERSION >= 3:
+            output = self.sampler.run_mcmc(p0, steps, progress=progress)
+            self._lastpos = output.coords
+        else:
+            output = self.sampler.run_mcmc(p0, steps)
+            self._lastpos = output[0]
 
         # discard the burn samples and thin
         chain = self.sampler.chain[..., burn::thin, :]
@@ -1163,6 +1295,36 @@ class Minimizer(object):
         result.lnprob = np.copy(lnprobability)
         result.errorbars = True
         result.nvarys = len(result.var_names)
+        result.nfev = ntemps*nwalkers*steps
+
+        # Calculate the residual with the "best fit" parameters
+        out = self.userfcn(params, *self.userargs, **self.userkws)
+        result.residual = _nan_policy(out, nan_policy=self.nan_policy, handle_inf=False)
+
+        # If uncertainty was automatically estimated, weight the residual properly
+        if (not is_weighted) and (result.residual.size > 1):
+            if '__lnsigma' in params:
+                result.residual = result.residual/np.exp(params['__lnsigma'].value)
+
+        # Calculate statistics for the two standard cases:
+        if isinstance(result.residual, ndarray) or (float_behavior == 'chi2'):
+            result._calculate_statistics()
+
+        # Handle special case unique to emcee:
+        # This should eventually be moved into result._calculate_statistics.
+        elif float_behavior == 'posterior':
+            result.ndata = 1
+            result.nfree = 1
+
+            # assuming prior prob = 1, this is true
+            _neg2_log_likel = -2*result.residual
+
+            # assumes that residual is properly weighted
+            result.chisqr = np.exp(_neg2_log_likel)
+
+            result.redchi = result.chisqr / result.nfree
+            result.aic = _neg2_log_likel + 2 * result.nvarys
+            result.bic = _neg2_log_likel + np.log(result.ndata) * result.nvarys
 
         if auto_pool is not None:
             auto_pool.terminate()
@@ -1212,19 +1374,25 @@ class Minimizer(object):
                                 bounds=(lower_bounds, upper_bounds),
                                 kwargs=dict(apply_bounds_transformation=False),
                                 **kws)
+            result.residual = ret.fun
         except AbortFitException:
             pass
 
+        result._calculate_statistics()
+
         if not result.aborted:
             for attr in ret:
                 setattr(result, attr, ret[attr])
 
             result.x = np.atleast_1d(result.x)
-            result.residual = ret.fun
-        else:
-            pass
 
-        result._calculate_statistics()
+            # calculate the cov_x and estimate uncertainties/correlations
+            try:
+                hess = np.matmul(ret.jac.T, ret.jac)
+                result.covar = np.linalg.inv(hess)
+                self._calculate_uncertainties_correlations()
+            except LinAlgError:
+                pass
 
         return result
 
@@ -1300,7 +1468,6 @@ class Minimizer(object):
             pass
 
         result._calculate_statistics()
-        params = result.params
 
         if result.aborted:
             return result
@@ -1321,82 +1488,18 @@ class Minimizer(object):
         else:
             result.message = 'Tolerance seems to be too small.'
 
-        # need to map _best values to params, then calculate the
-        # grad for the variable parameters
-        grad = ones_like(_best)
-        vbest = ones_like(_best)
-
-        # ensure that _best, vbest, and grad are not
-        # broken 1-element ndarrays.
-        if len(np.shape(_best)) == 0:
-            _best = np.array([_best])
-        if len(np.shape(vbest)) == 0:
-            vbest = np.array([vbest])
-        if len(np.shape(grad)) == 0:
-            grad = np.array([grad])
-
-        for ivar, name in enumerate(result.var_names):
-            grad[ivar] = params[name].scale_gradient(_best[ivar])
-            vbest[ivar] = params[name].value
-
-        # modified from JJ Helmus' leastsqbound.py
-        infodict['fjac'] = transpose(transpose(infodict['fjac']) /
-                                     take(grad, infodict['ipvt'] - 1))
-        rvec = dot(triu(transpose(infodict['fjac'])[:nvars, :]),
-                   take(eye(nvars), infodict['ipvt'] - 1, 0))
-        try:
-            result.covar = inv(dot(transpose(rvec), rvec))
-        except (LinAlgError, ValueError):
-            result.covar = None
-
-        result.fjac = infodict['fjac']
-
-        has_expr = False
-        for par in params.values():
-            par.stderr, par.correl = 0, None
-            has_expr = has_expr or par.expr is not None
-
         # self.errorbars = error bars were successfully estimated
-        result.errorbars = (result.covar is not None)
+        result.errorbars = (_cov is not None)
         if result.errorbars:
-            if self.scale_covar:
-                result.covar *= result.redchi
-            for ivar, name in enumerate(result.var_names):
-                par = params[name]
-                par.stderr = sqrt(result.covar[ivar, ivar])
-                par.correl = {}
-                try:
-                    result.errorbars = result.errorbars and (par.stderr > 0.0)
-                    for jvar, varn2 in enumerate(result.var_names):
-                        if jvar != ivar:
-                            par.correl[varn2] = (
-                                result.covar[ivar, jvar] /
-                                (par.stderr * sqrt(result.covar[jvar, jvar])))
-                # TODO: do not use bare except
-                except:
-                    result.errorbars = False
-
-            if has_expr:
-                # uncertainties on constrained parameters:
-                #   get values with uncertainties (including correlations),
-                #   temporarily set Parameter values to these,
-                #   re-evaluate contrained parameters to extract stderr
-                #   and then set Parameters back to best-fit value
-                try:
-                    uvars = uncertainties.correlated_values(vbest, result.covar)
-                except (LinAlgError, ValueError):
-                    uvars = None
-                if uvars is not None:
-                    for par in params.values():
-                        eval_stderr(par, uvars, result.var_names, params)
-                    # restore nominal values
-                    for v, nam in zip(uvars, result.var_names):
-                        params[nam].value = v.nominal_value
-
-        if not result.errorbars:
+            # transform the covariance matrix to "external" parameter space
+            result.covar = self._int2ext_cov_x(_cov, _best)
+            # calculate parameter uncertainties and correlations
+            self._calculate_uncertainties_correlations()
+        else:
             result.message = '%s Could not estimate error-bars.' % result.message
 
         np.seterr(**orig_warn_settings)
+
         return result
 
     def basinhopping(self, params=None, **kws):
@@ -1435,8 +1538,8 @@ class Minimizer(object):
         basinhopping_kws.update(kws)
 
         # FIXME - remove after requirement for scipy >= 0.19
-        major, minor, micro = np.array(scipy_version.split('.'), dtype='int')
-        if major < 1 and minor < 19:
+        major, minor, micro = scipy_version.split('.', 2)
+        if int(major) < 1 and int(minor) < 19:
             _ = basinhopping_kws.pop('seed')
             print("Warning: basinhopping doesn't support argument 'seed' for "
                   "scipy versions below 0.19!")
@@ -1452,11 +1555,17 @@ class Minimizer(object):
             result.message = ret.message
             result.residual = self.__residual(ret.x)
             result.nfev -= 1
-        else:
-            pass
 
         result._calculate_statistics()
 
+        # calculate the cov_x and estimate uncertanties/correlations
+        if (not result.aborted and self.calc_covar and HAS_NUMDIFFTOOLS and
+                len(result.residual) > len(result.var_names)):
+            _covar_ndt = self._calculate_covariance_matrix(ret.x)
+            if _covar_ndt is not None:
+                result.covar = self._int2ext_cov_x(_covar_ndt, ret.x)
+                self._calculate_uncertainties_correlations()
+
         return result
 
     def brute(self, params=None, Ns=20, keep=50):
@@ -1614,16 +1723,16 @@ class Minimizer(object):
             result.params = result.candidates[0].params
             result.residual = self.__residual(result.brute_x0, apply_bounds_transformation=False)
             result.nfev -= 1
-        else:
-            pass
 
         result._calculate_statistics()
 
         return result
 
     def ampgo(self, params=None, **kws):
-        """Finds the global minimum of a multivariate function using the AMPGO
-        (Adaptive Memory Programming for Global Optimization) algorithm.
+        """Find the global minimum of a multivariate function using AMPGO.
+
+        AMPGO stands for 'Adaptive Memory Programming for Global Optimization'
+        and is an efficient algorithm to find the global minimum.
 
         Parameters
         ----------
@@ -1719,11 +1828,17 @@ class Minimizer(object):
 
             result.residual = self.__residual(result.ampgo_x0)
             result.nfev -= 1
-        else:
-            pass
 
         result._calculate_statistics()
 
+        # calculate the cov_x and estimate uncertanties/correlations
+        if (not result.aborted and self.calc_covar and HAS_NUMDIFFTOOLS and
+                len(result.residual) > len(result.var_names)):
+            _covar_ndt = self._calculate_covariance_matrix(result.ampgo_x0)
+            if _covar_ndt is not None:
+                result.covar = self._int2ext_cov_x(_covar_ndt, result.ampgo_x0)
+                self._calculate_uncertainties_correlations()
+
         return result
 
     def minimize(self, method='leastsq', params=None, **kws):
@@ -1735,7 +1850,7 @@ class Minimizer(object):
             Name of the fitting method to use. Valid values are:
 
             - `'leastsq'`: Levenberg-Marquardt (default)
-            - `'least_squares'`: Least-Squares minimization, using Trust Region Reflective method by default
+            - `'least_squares'`: Least-Squares minimization, using Trust Region Reflective method
             - `'differential_evolution'`: differential evolution
             - `'brute'`: brute force method
             - `'basinhopping'`: basinhopping
@@ -1754,6 +1869,7 @@ class Minimizer(object):
             - `'trust-constr'`: trust-region for constrained optimization (SciPy >= 1.1)
             - `'dogleg'`: Dog-leg trust-region
             - `'slsqp'`: Sequential Linear Squares Programming
+            - `'emcee'`: Maximum likelihood via Monte-Carlo Markov Chain
 
             In most cases, these methods wrap and use the method with the
             same name from `scipy.optimize`, or use
@@ -1799,6 +1915,8 @@ class Minimizer(object):
             function = self.basinhopping
         elif user_method == 'ampgo':
             function = self.ampgo
+        elif user_method == 'emcee':
+            function = self.emcee
         else:
             function = self.scalar_minimize
             for key, val in SCALAR_METHODS.items():
@@ -1945,8 +2063,12 @@ VALID_NAN_POLICIES = ('propagate', 'omit', 'raise')
 
 
 def validate_nan_policy(policy):
-    """Validate, rationalize nan_policy, for backward compatibility and
-    compatibility with Pandas missing convention."""
+    """Validate and rationalize `nan_policy`.
+
+    This function ensures backwards compatibility and as well as compatibility
+    with Pandas `missing` convention.
+
+    """
     if policy in VALID_NAN_POLICIES:
         return policy
     if policy is None:
@@ -2019,8 +2141,9 @@ def _nan_policy(arr, nan_policy='raise', handle_inf=True):
     return arr
 
 
-def minimize(fcn, params, method='leastsq', args=None, kws=None,
-             scale_covar=True, iter_cb=None, reduce_fcn=None, **fit_kws):
+def minimize(fcn, params, method='leastsq', args=None, kws=None, iter_cb=None,
+             scale_covar=True, nan_policy='raise', reduce_fcn=None,
+             calc_covar=True, **fit_kws):
     """Perform a fit of a set of parameters by minimizing an objective (or
     cost) function using one of the several available methods.
 
@@ -2044,7 +2167,7 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
         Name of the fitting method to use. Valid values are:
 
         - `'leastsq'`: Levenberg-Marquardt (default)
-        - `'least_squares'`: Least-Squares minimization, using Trust Region Reflective method by default
+        - `'least_squares'`: Least-Squares minimization, using Trust Region Reflective method
         - `'differential_evolution'`: differential evolution
         - `'brute'`: brute force method
         - `'basinhopping'`: basinhopping
@@ -2063,6 +2186,7 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
         - `'trust-constr'`: trust-region for constrained optimization (SciPy >= 1.1)
         - `'dogleg'`: Dog-leg trust-region
         - `'slsqp'`: Sequential Linear Squares Programming
+        - `'emcee'`: Maximum likelihood via Monte-Carlo Markov Chain
 
         In most cases, these methods wrap and use the method of the same
         name from `scipy.optimize`, or use `scipy.optimize.minimize` with
@@ -2084,11 +2208,22 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
         the iteration number, `resid` the current residual array, and `*args`
         and `**kws` as passed to the objective function.
     scale_covar : bool, optional
-        Whether to automatically scale the covariance matrix (default is True,
-        `leastsq` only).
+        Whether to automatically scale the covariance matrix (default is True).
+    nan_policy : str, optional
+        Specifies action if `userfcn` (or a Jacobian) returns NaN
+        values. One of:
+
+        - 'raise' : a `ValueError` is raised
+        - 'propagate' : the values returned from `userfcn` are un-altered
+        - 'omit' : non-finite values are filtered
+
     reduce_fcn : str or callable, optional
         Function to convert a residual array to a scalar value for the scalar
         minimizers. See notes in `Minimizer`.
+    calc_covar : bool, optional
+        Whether to calculate the covariance matrix (default is True) for
+        solvers other than `leastsq` and `least_squares`. Requires the
+        `numdifftools` package to be installed.
     **fit_kws : dict, optional
         Options to pass to the minimizer being used.
 
@@ -2126,11 +2261,14 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     and is equivalent to::
 
         fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
-                           iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
+                           iter_cb=iter_cb, scale_covar=scale_covar,
+                           nan_policy=nan_policy, reduce_fcn=reduce_fcn,
+                           calc_covar=calc_covar, **fit_kws)
         fitter.minimize(method=method)
 
     """
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar,
-                       reduce_fcn=reduce_fcn, **fit_kws)
+                       nan_policy=nan_policy, reduce_fcn=reduce_fcn,
+                       calc_covar=calc_covar, **fit_kws)
     return fitter.minimize(method=method)
