@@ -1,6 +1,4 @@
-"""Concise nonlinear curve fitting."""
-from __future__ import print_function
-
+"""Implementation of the Model interface."""
 from collections import OrderedDict
 from copy import deepcopy
 from functools import wraps
@@ -56,7 +54,136 @@ def _ensureMatplotlib(function):
     return no_op
 
 
+def get_reducer(option):
+    """Factory function to build a parser for complex numbers.
+
+    Parameters
+    ----------
+    option : str
+        Should be one of `['real', 'imag', 'abs', 'angle']`. Implements the
+        numpy function of the same name.
+
+    Returns
+    -------
+    callable
+        See docstring for `reducer` below.
+
+    """
+    if option not in ['real', 'imag', 'abs', 'angle']:
+        raise ValueError("Invalid parameter name ('%s') for function 'propagate_err'." % option)
+
+    def reducer(array):
+        """Convert a complex array to a real array.
+
+        Several conversion methods are available and it does nothing to a
+        purely real array.
+
+        Parameters
+        ----------
+        array : array-like
+            Input array. If complex, will be converted to real array via one
+            of the following numpy functions: `real`, `imag`, `abs`, or `angle`.
+
+        Returns
+        -------
+        numpy.array
+            Returned array will be purely real.
+
+        """
+        if any(np.iscomplex(array)):
+            parsed_array = getattr(np, option)(array)
+        else:
+            parsed_array = array
+
+        return parsed_array
+    return reducer
+
+
+def propagate_err(z, dz, option):
+    """Perform error propagation on a vector of complex uncertainties to
+    get values for magnitude (abs) and phase (angle) uncertainty.
+
+    Parameters
+    ----------
+    z : array-like
+        Array of complex or real numbers.
+
+    dz : array-like
+        Array of uncertainties corresponding to z. Must satisfy
+        `numpy.shape(dz) == np.shape(z)`.
+
+    option : str
+        Should be one of `['real', 'imag', 'abs', 'angle']`.
+
+    Returns
+    -------
+    numpy.array
+        Returned array will be purely real.
+
+    Notes
+    -----
+    Uncertainties are 1/weights. If the weights provided are real, they are
+    assumed to apply equally to the real and imaginary parts. If the weights
+    are complex, the real part of the weights are applied to the real
+    part of the residual and the imaginary part is treated correspondingly.
+
+    In the case where `option == 'angle'` and `numpy.abs(z) == 0` for any value
+    of `z` the phase angle uncertainty becomes the entire circle and so
+    a value of pi is returned.
+
+    In the case where `option == 'abs'` and `numpy.abs(z) == 0` for any value of
+    `z` the mangnitude uncertainty is approximated by `numpy.abs(dz)` for that
+    value.
+
+    """
+    if option not in ['real', 'imag', 'abs', 'angle']:
+        raise ValueError("Invalid parameter name ('%s') for function 'propagate_err'." % option)
+
+    # Check the main vector for complex. Do nothing if real.
+    if any(np.iscomplex(z)):
+        # if uncertainties are real, apply them equally to
+        # real and imaginary parts
+        if all(np.isreal(dz)):
+            dz = dz+1j*dz
+
+        if option == 'real':
+            err = np.real(dz)
+        elif option == 'imag':
+            err = np.imag(dz)
+        elif option in ['abs', 'angle']:
+            rz = np.real(z)
+            iz = np.imag(z)
+
+            rdz = np.real(dz)
+            idz = np.imag(dz)
+
+            # Don't spit out warnings for divide by zero. Will fix these later.
+            with np.errstate(divide='ignore', invalid='ignore'):
+
+                if option == 'abs':
+                    # Standard error propagation for abs = sqrt(re**2 + im**2)
+                    err = np.true_divide(np.sqrt((iz*idz)**2+(rz*rdz)**2),
+                                         np.abs(z))
+
+                    # For abs = 0, error is +/- abs(rdz + j idz)
+                    err[err == np.inf] = np.abs(dz)[err == np.inf]
+
+                if option == 'angle':
+                    # Standard error propagation for angle = arctan(im/re)
+                    err = np.true_divide(np.sqrt((rz*idz)**2+(iz*rdz)**2),
+                                         np.abs(z)**2)
+
+                    # For abs = 0, error is +/- pi (i.e. the whole circle)
+                    err[err == np.inf] = np.pi
+    else:
+        err = dz
+
+    return err
+
+
 class Model(object):
+    """Model class."""
+
     _forbidden_args = ('data', 'weights', 'params')
     _invalid_ivar = "Invalid independent variable name ('%s') for function %s"
     _invalid_par = "Invalid parameter name ('%s') for function %s"
@@ -192,8 +319,8 @@ class Model(object):
         Note: like the standard-ish '__setstate__' method but not really
         useful with Pickle.
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         state :
             Serialized state from `_get_state`.
         funcdefs : dict, optional
@@ -317,8 +444,7 @@ class Model(object):
         self._parse_params()
 
     def _set_paramhints_prefix(self):
-        """Reset parameter hints for prefix:
-        intended to be overwritten"""
+        """Reset parameter hints for prefix: intended to be overwritten."""
         pass
 
     @property
@@ -732,7 +858,7 @@ class Model(object):
 
     def fit(self, data, params=None, weights=None, method='leastsq',
             iter_cb=None, scale_covar=True, verbose=False, fit_kws=None,
-            nan_policy=None, **kwargs):
+            nan_policy=None, calc_covar=True, **kwargs):
         """Fit the model to the data using the supplied Parameters.
 
         Parameters
@@ -750,7 +876,7 @@ class Model(object):
             Callback function to call at each iteration (default is None).
         scale_covar : bool, optional
             Whether to automatically scale the covariance matrix when
-            calculating uncertainties (default is True, `leastsq` method only).
+            calculating uncertainties (default is True).
         verbose: bool, optional
             Whether to print a message when a new parameter is added because
             of a hint (default is True).
@@ -758,6 +884,10 @@ class Model(object):
             What to do when encountering NaNs when fitting Model.
         fit_kws: dict, optional
             Options to pass to the minimizer being used.
+        calc_covar : bool, optional
+            Whether to calculate the covariance matrix (default is True) for
+            solvers other than `leastsq` and `least_squares`. Requires the
+            `numdifftools` package to be installed.
         **kwargs: optional
             Arguments to pass to the  model function, possibly overriding
             params.
@@ -841,7 +971,7 @@ class Model(object):
             data = np.asfarray(data)
         for var in self.independent_vars:
             var_data = kwargs[var]
-            if (not hasattr(var_data, '__array__')) and (not np.isscalar(var_data)):
+            if isinstance(var_data, (list, tuple)):
                 kwargs[var] = np.asfarray(var_data)
 
         # Handle null/missing values.
@@ -869,7 +999,8 @@ class Model(object):
 
         output = ModelResult(self, params, method=method, iter_cb=iter_cb,
                              scale_covar=scale_covar, fcn_kws=kwargs,
-                             nan_policy=self.nan_policy, **fit_kws)
+                             nan_policy=self.nan_policy, calc_covar=calc_covar,
+                             **fit_kws)
         output.fit(data=data, weights=weights)
         output.components = self.components
         return output
@@ -1121,11 +1252,14 @@ def load_modelresult(fname, funcdefs=None):
     ModelResult
 
     """
-    p = Parameters()
-    m = ModelResult(Model(lambda x: x, None), p)
+    params = Parameters()
+    modres = ModelResult(Model(lambda x: x, None), params)
     with open(fname) as fh:
-        modelresult = m.load(fh, funcdefs=funcdefs)
-    return modelresult
+        mresult = modres.load(fh, funcdefs=funcdefs)
+        mresult.data = mresult.userargs[0]
+        mresult.weights = mresult.userargs[1]
+        mresult.init_params = mresult.model.make_params(**mresult.init_values)
+    return mresult
 
 
 class ModelResult(Minimizer):
@@ -1140,7 +1274,7 @@ class ModelResult(Minimizer):
     def __init__(self, model, params, data=None, weights=None,
                  method='leastsq', fcn_args=None, fcn_kws=None,
                  iter_cb=None, scale_covar=True, nan_policy='raise',
-                 **fit_kws):
+                 calc_covar=True, **fit_kws):
         """
         Parameters
         ----------
@@ -1164,6 +1298,10 @@ class ModelResult(Minimizer):
             Whether to scale covariance matrix for uncertainty evaluation.
         nan_policy : str, optional, one of 'raise' (default), 'propagate', or 'omit'.
             What to do when encountering NaNs when fitting Model.
+        calc_covar : bool, optional
+            Whether to calculate the covariance matrix (default is True) for
+            solvers other than `leastsq` and `least_squares`. Requires the
+            `numdifftools` package to be installed.
         **fit_kws : optional
             Keyword arguments to send to minimization routine.
 
@@ -1177,7 +1315,7 @@ class ModelResult(Minimizer):
         self.init_params = deepcopy(params)
         Minimizer.__init__(self, model._residual, params, fcn_args=fcn_args,
                            fcn_kws=fcn_kws, iter_cb=iter_cb, nan_policy=nan_policy,
-                           scale_covar=scale_covar, **fit_kws)
+                           scale_covar=scale_covar, calc_covar=calc_covar, **fit_kws)
 
     def fit(self, data=None, params=None, weights=None, method=None,
             nan_policy=None, **kwargs):
@@ -1243,10 +1381,11 @@ class ModelResult(Minimizer):
            Array for evaluated model.
 
         """
-        self.userkws.update(kwargs)
+        userkws = self.userkws.copy()
+        userkws.update(kwargs)
         if params is None:
             params = self.params
-        return self.model.eval(params=params, **self.userkws)
+        return self.model.eval(params=params, **userkws)
 
     def eval_components(self, params=None, **kwargs):
         """Evaluate each component of a composite model function.
@@ -1265,22 +1404,24 @@ class ModelResult(Minimizer):
              the estimated model value for each component of the model.
 
         """
-        self.userkws.update(kwargs)
+        userkws = self.userkws.copy()
+        userkws.update(kwargs)
         if params is None:
             params = self.params
-        return self.model.eval_components(params=params, **self.userkws)
+        return self.model.eval_components(params=params, **userkws)
 
     def eval_uncertainty(self, params=None, sigma=1, **kwargs):
-        """Evaluate the uncertainty of the *model function* from the
-        uncertainties for the best-fit parameters.  This can be used to give
-        confidence bands for the model.
+        """Evaluate the uncertainty of the *model function*.
+
+        This can be used to give confidence bands for the model from the
+        uncertainties in the best-fit parameters.
 
         Parameters
         ----------
         params : Parameters, optional
              Parameters, defaults to ModelResult.params.
         sigma : float, optional
-             Confidence level, i.e. how many sigma (default  is 1).
+             Confidence level, i.e. how many sigma (default is 1).
         **kwargs : optional
              Values of options, independent variables, etcetera.
 
@@ -1304,12 +1445,12 @@ class ModelResult(Minimizer):
         1. This is based on the excellent and clear example from
            https://www.astro.rug.nl/software/kapteyn/kmpfittutorial.html#confidence-and-prediction-intervals,
            which references the original work of:
-           J. Wolberg,Data Analysis Using the Method of Least Squares, 2006, Springer
-        2. The value of sigma is number of `sigma` values, and is converted to a
-           probability.  Values or 1, 2, or 3 give probalities of 0.6827, 0.9545,
-           and 0.9973, respectively. If the sigma value is < 1, it is interpreted
-           as the probability itself.  That is, `sigma=1` and `sigma=0.6827` will
-           give the same results, within precision errors.
+           J. Wolberg, Data Analysis Using the Method of Least Squares, 2006, Springer
+        2. The value of sigma is number of `sigma` values, and is converted to
+           a probability. Values of 1, 2, or 3 give probabilities of 0.6827,
+           0.9545, and 0.9973, respectively. If the sigma value is < 1, it is
+           interpreted as the probability itself. That is, `sigma=1` and
+           `sigma=0.6827` will give the same results, within precision errors.
 
         """
         userkws = self.userkws.copy()
@@ -1446,21 +1587,21 @@ class ModelResult(Minimizer):
                'model': encode4js(self.model._get_state())}
         pasteval = self.params._asteval
         out['params'] = [p.__getstate__() for p in self.params.values()]
-        out['unique_symbols'] = {key: pasteval.symtable[key]
+        out['unique_symbols'] = {key: encode4js(pasteval.symtable[key])
                                  for key in pasteval.user_defined_symbols()}
 
         for attr in ('aborted', 'aic', 'best_values', 'bic', 'chisqr',
                      'ci_out', 'col_deriv', 'covar', 'errorbars',
                      'flatchain', 'ier', 'init_values', 'lmdif_message',
                      'message', 'method', 'nan_policy', 'ndata', 'nfev',
-                     'nfree', 'nvarys', 'redchi', 'scale_covar', 'success',
-                     'userargs', 'userkws', 'values', 'var_names',
+                     'nfree', 'nvarys', 'redchi', 'scale_covar', 'calc_covar',
+                     'success', 'userargs', 'userkws', 'values', 'var_names',
                      'weights', 'user_options'):
             val = getattr(self, attr)
             if isinstance(val, np.bool_):
                 val = bool(val)
             out[attr] = encode4js(val)
-        return json.dumps(out)
+        return json.dumps(out, **kws)
 
     def dump(self, fp, **kws):
         """Dump serialization of ModelResult to a file.
@@ -1533,8 +1674,8 @@ class ModelResult(Minimizer):
                      'init_values', 'kws', 'lmdif_message', 'message',
                      'method', 'nan_policy', 'ndata', 'nfev', 'nfree',
                      'nvarys', 'redchi', 'residual', 'scale_covar',
-                     'success', 'userargs', 'userkws', 'var_names',
-                     'weights', 'user_options'):
+                     'calc_covar', 'success', 'userargs', 'userkws',
+                     'var_names', 'weights', 'user_options'):
             setattr(self, attr, decode4js(modres.get(attr, None)))
 
         self.best_fit = self.model.eval(self.params, **self.userkws)
@@ -1568,7 +1709,7 @@ class ModelResult(Minimizer):
     def plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--',
                  xlabel=None, ylabel=None, yerr=None, numpoints=None,
                  data_kws=None, fit_kws=None, init_kws=None, ax_kws=None,
-                 show_init=False):
+                 show_init=False, parse_complex='abs'):
         """Plot the fit results using matplotlib, if available.
 
         The plot will include the data points, the initial fit curve (optional,
@@ -1609,6 +1750,10 @@ class ModelResult(Minimizer):
             Keyword arguments for a new axis, if there is one being created.
         show_init : bool, optional
             Whether to show the initial conditions for the fit (default is False).
+        parse_complex : str, optional
+            How to reduce complex data for plotting.
+            Options are one of `['real', 'imag', 'abs', 'angle']`, which
+            correspond to the numpy functions of the same name (default is 'abs').
 
         Returns
         -------
@@ -1623,6 +1768,9 @@ class ModelResult(Minimizer):
         matplotlib.axes.Axes.errorbar is used to plot the data.  If `yerr` is
         not specified and the fit includes weights, `yerr` set to 1/self.weights
 
+        If model returns complex data, `yerr` is treated the same way that
+        weights are in this case.
+
         If `ax` is None then `matplotlib.pyplot.gca(**ax_kws)` is called.
 
         See Also
@@ -1640,6 +1788,9 @@ class ModelResult(Minimizer):
         if ax_kws is None:
             ax_kws = {}
 
+        # The function reduce_complex will convert complex vectors into real vectors
+        reduce_complex = get_reducer(parse_complex)
+
         if len(self.model.independent_vars) == 1:
             independent_var = self.model.independent_vars[0]
         else:
@@ -1661,21 +1812,25 @@ class ModelResult(Minimizer):
         if show_init:
             ax.plot(
                 x_array_dense,
-                self.model.eval(self.init_params,
-                                **{independent_var: x_array_dense}),
+                reduce_complex(self.model.eval(
+                    self.init_params, **{independent_var: x_array_dense})),
                 initfmt, label='init', **init_kws)
 
         if yerr is None and self.weights is not None:
             yerr = 1.0/self.weights
+
         if yerr is not None:
-            ax.errorbar(x_array, self.data, yerr=yerr,
+            ax.errorbar(x_array, reduce_complex(self.data),
+                        yerr=propagate_err(self.data, yerr, parse_complex),
                         fmt=datafmt, label='data', **data_kws)
         else:
-            ax.plot(x_array, self.data, datafmt, label='data', **data_kws)
+            ax.plot(x_array, reduce_complex(self.data),
+                    datafmt, label='data', **data_kws)
 
         ax.plot(
             x_array_dense,
-            self.model.eval(self.params, **{independent_var: x_array_dense}),
+            reduce_complex(self.model.eval(self.params,
+                                           **{independent_var: x_array_dense})),
             fitfmt, label='best-fit', **fit_kws)
 
         ax.set_title(self.model.name)
@@ -1692,7 +1847,7 @@ class ModelResult(Minimizer):
 
     @_ensureMatplotlib
     def plot_residuals(self, ax=None, datafmt='o', yerr=None, data_kws=None,
-                       fit_kws=None, ax_kws=None):
+                       fit_kws=None, ax_kws=None, parse_complex='abs'):
         """Plot the fit residuals using matplotlib, if available.
 
         If `yerr` is supplied or if the model included weights, errorbars
@@ -1713,6 +1868,10 @@ class ModelResult(Minimizer):
             Keyword arguments passed on to the plot function for fitted curve.
         ax_kws : dict, optional
             Keyword arguments for a new axis, if there is one being created.
+        parse_complex : str, optional
+            How to reduce complex data for plotting.
+            Options are one of `['real', 'imag', 'abs', 'angle']`, which
+            correspond to the numpy functions of the same name (default is 'abs').
 
         Returns
         -------
@@ -1727,6 +1886,9 @@ class ModelResult(Minimizer):
         matplotlib.axes.Axes.errorbar is used to plot the data.  If `yerr` is
         not specified and the fit includes weights, `yerr` set to 1/self.weights
 
+        If model returns complex data, `yerr` is treated the same way that
+        weights are in this case.
+
         If `ax` is None then `matplotlib.pyplot.gca(**ax_kws)` is called.
 
         See Also
@@ -1742,6 +1904,9 @@ class ModelResult(Minimizer):
         if ax_kws is None:
             ax_kws = {}
 
+        # The function reduce_complex will convert complex vectors into real vectors
+        reduce_complex = get_reducer(parse_complex)
+
         if len(self.model.independent_vars) == 1:
             independent_var = self.model.independent_vars[0]
         else:
@@ -1759,10 +1924,11 @@ class ModelResult(Minimizer):
         if yerr is None and self.weights is not None:
             yerr = 1.0/self.weights
         if yerr is not None:
-            ax.errorbar(x_array, self.eval() - self.data, yerr=yerr,
+            ax.errorbar(x_array, reduce_complex(self.eval()) - reduce_complex(self.data),
+                        yerr=propagate_err(self.data, yerr, parse_complex),
                         fmt=datafmt, label='residuals', **data_kws)
         else:
-            ax.plot(x_array, self.eval() - self.data, datafmt,
+            ax.plot(x_array, reduce_complex(self.eval()) - reduce_complex(self.data), datafmt,
                     label='residuals', **data_kws)
 
         ax.set_title(self.model.name)
@@ -1774,7 +1940,7 @@ class ModelResult(Minimizer):
     def plot(self, datafmt='o', fitfmt='-', initfmt='--', xlabel=None,
              ylabel=None, yerr=None, numpoints=None, fig=None, data_kws=None,
              fit_kws=None, init_kws=None, ax_res_kws=None, ax_fit_kws=None,
-             fig_kws=None, show_init=False):
+             fig_kws=None, show_init=False, parse_complex='abs'):
         """Plot the fit results and residuals using matplotlib, if available.
 
         The method will produce a matplotlib figure with both results of the
@@ -1818,6 +1984,11 @@ class ModelResult(Minimizer):
             Keyword arguments for a new figure, if there is one being created.
         show_init : bool, optional
             Whether to show the initial conditions for the fit (default is False).
+        parse_complex : str, optional
+            How to reduce complex data for plotting.
+            Options are one of `['real', 'imag', 'abs', 'angle']`, which
+            correspond to the numpy functions of the same name (default is 'abs').
+
 
         Returns
         -------
@@ -1831,6 +2002,9 @@ class ModelResult(Minimizer):
         matplotlib.axes.Axes.errorbar is used to plot the data.  If `yerr` is
         not specified and the fit includes weights, `yerr` set to 1/self.weights
 
+        If model returns complex data, `yerr` is treated the same way that
+        weights are in this case.
+
         If `fig` is None then `matplotlib.pyplot.figure(**fig_kws)` is called,
         otherwise `fig_kws` is ignored.
 
@@ -1850,6 +2024,7 @@ class ModelResult(Minimizer):
             ax_res_kws = {}
         if ax_fit_kws is None:
             ax_fit_kws = {}
+
         # make a square figure with side equal to the default figure's x-size
         figxsize = plt.rcParams['figure.figsize'][0]
         fig_kws_ = dict(figsize=(figxsize, figxsize))
@@ -1872,10 +2047,10 @@ class ModelResult(Minimizer):
                       initfmt=initfmt, xlabel=xlabel, ylabel=ylabel,
                       numpoints=numpoints, data_kws=data_kws,
                       fit_kws=fit_kws, init_kws=init_kws, ax_kws=ax_fit_kws,
-                      show_init=show_init)
+                      show_init=show_init, parse_complex=parse_complex)
         self.plot_residuals(ax=ax_res, datafmt=datafmt, yerr=yerr,
                             data_kws=data_kws, fit_kws=fit_kws,
-                            ax_kws=ax_res_kws)
+                            ax_kws=ax_res_kws, parse_complex=parse_complex)
         plt.setp(ax_res.get_xticklabels(), visible=False)
         ax_fit.set_title('')
         return fig, gs
