@@ -1,19 +1,21 @@
 """Module containing built-in fitting models."""
-import numpy as np
+import time
 
 from asteval import Interpreter, get_ast_names
+import numpy as np
 
 from . import lineshapes
 from .lineshapes import (breit_wigner, damped_oscillator, dho, donaich,
                          expgaussian, exponential, gaussian, linear, lognormal,
                          lorentzian, moffat, parabolic, pearson7, powerlaw,
-                         pvoigt, rectangle, skewed_gaussian, step, students_t,
-                         voigt)
+                         pvoigt, rectangle, skewed_gaussian, skewed_voigt,
+                         step, students_t, voigt)
 from .model import Model
 
 
 class DimensionalError(Exception):
-    """TODO: class docstring."""
+    """Raise exception when number of independent variables is not one."""
+
     pass
 
 
@@ -147,6 +149,7 @@ class ConstantModel(Model):
         super(ConstantModel, self).__init__(constant, **kwargs)
 
     def guess(self, data, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = self.make_params()
 
         pars['%sc' % self.prefix].set(value=data.mean())
@@ -176,6 +179,7 @@ class ComplexConstantModel(Model):
         super(ComplexConstantModel, self).__init__(constant, **kwargs)
 
     def guess(self, data, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = self.make_params()
         pars['%sre' % self.prefix].set(value=data.real.mean())
         pars['%sim' % self.prefix].set(value=data.imag.mean())
@@ -205,6 +209,7 @@ class LinearModel(Model):
         super(LinearModel, self).__init__(linear, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         sval, oval = 0., 0.
         if x is not None:
             sval, oval = np.polyfit(x, data, 1)
@@ -233,6 +238,7 @@ class QuadraticModel(Model):
         super(QuadraticModel, self).__init__(parabolic, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         a, b, c = 0., 0., 0.
         if x is not None:
             a, b, c = np.polyfit(x, data, 2)
@@ -279,6 +285,7 @@ class PolynomialModel(Model):
         super(PolynomialModel, self).__init__(polynomial, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = self.make_params()
         if x is not None:
             out = np.polyfit(x, data, self.poly_degree)
@@ -324,6 +331,7 @@ class GaussianModel(Model):
         self.set_param_hint('height', expr=height_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -364,6 +372,7 @@ class LorentzianModel(Model):
         self.set_param_hint('height', expr=height_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative, ampscale=1.25)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -403,9 +412,6 @@ class VoigtModel(Model):
     the full width at half maximum is approximately :math:`3.6013\sigma`.
 
     """
-
-    fwhm_factor = 3.60131
-
     def __init__(self, independent_vars=['x'], prefix='', nan_policy='raise',
                  **kwargs):
         kwargs.update({'prefix': prefix, 'nan_policy': nan_policy,
@@ -416,13 +422,17 @@ class VoigtModel(Model):
     def _set_paramhints_prefix(self):
         self.set_param_hint('sigma', min=0)
         self.set_param_hint('gamma', expr='%ssigma' % self.prefix)
-        self.set_param_hint('fwhm', expr=fwhm_expr(self))
 
-        fmt = ("{prefix:s}amplitude*wofz((1j*{prefix:s}gamma)/"
-               "({prefix:s}sigma*sqrt(2))).real/({prefix:s}sigma*sqrt(2*pi))")
-        self.set_param_hint('height', expr=fmt.format(prefix=self.prefix))
+        fexpr = ("1.0692*{pre:s}gamma+" +
+                 "sqrt(0.8664*{pre:s}gamma**2+5.545083*{pre:s}sigma**2)")
+        hexpr = ("({pre:s}amplitude/({pre:s}sigma*sqrt(2*pi)))*"
+                 "wofz((1j*{pre:s}gamma)/({pre:s}sigma*sqrt(2))).real")
+
+        self.set_param_hint('fwhm', expr=fexpr.format(pre=self.prefix))
+        self.set_param_hint('height', expr=hexpr.format(pre=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative,
                                ampscale=1.5, sigscale=0.65)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -473,6 +483,7 @@ class PseudoVoigtModel(Model):
         self.set_param_hint('height', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative, ampscale=1.25)
         pars['%sfraction' % self.prefix].set(value=0.5, min=0.0, max=1.0)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -514,6 +525,7 @@ class MoffatModel(Model):
         self.set_param_hint('height', expr="%samplitude" % self.prefix)
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative, ampscale=0.5, sigscale=1.)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -557,6 +569,7 @@ class Pearson7Model(Model):
         self.set_param_hint('height', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         pars['%sexpon' % self.prefix].set(value=1.5)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -598,6 +611,7 @@ class StudentsTModel(Model):
         self.set_param_hint('fwhm', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -635,6 +649,7 @@ class BreitWignerModel(Model):
         self.set_param_hint('fwhm', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         pars['%sq' % self.prefix].set(value=1.0)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -678,6 +693,7 @@ class LognormalModel(Model):
         self.set_param_hint('fwhm', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = self.make_params(amplitude=1.0, center=0.0, sigma=0.25)
         pars['%ssigma' % self.prefix].set(min=0.0)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -721,6 +737,7 @@ class DampedOscillatorModel(Model):
         self.set_param_hint('fwhm', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative,
                                ampscale=0.1, sigscale=0.1)
         return update_param_vals(pars, self.prefix, **kwargs)
@@ -768,6 +785,7 @@ class DampedHarmonicOscillatorModel(Model):
         self.set_param_hint('fwhm', expr=fwhm_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative,
                                ampscale=0.1, sigscale=0.1)
         pars['%sgamma' % self.prefix].set(value=1.0, min=0.0)
@@ -815,6 +833,7 @@ class ExponentialGaussianModel(Model):
         self.set_param_hint('fwhm', expr=fwhm_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -835,7 +854,7 @@ class SkewedGaussianModel(Model):
        f(x; A, \mu, \sigma, \gamma) = \frac{A}{\sigma\sqrt{2\pi}}
        e^{[{-{(x-\mu)^2}/{{2\sigma}^2}}]} \Bigl\{ 1 +
        {\operatorname{erf}}\bigl[
-       \frac{\gamma(x-\mu)}{\sigma\sqrt{2}}
+       \frac{{\gamma}(x-\mu)}{\sigma\sqrt{2}}
        \bigr] \Bigr\}
 
     where :func:`erf` is the error function.
@@ -858,6 +877,55 @@ class SkewedGaussianModel(Model):
         self.set_param_hint('fwhm', expr=fwhm_expr(self))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
+        pars = guess_from_peak(self, data, x, negative)
+        return update_param_vals(pars, self.prefix, **kwargs)
+
+    __init__.__doc__ = COMMON_INIT_DOC
+    guess.__doc__ = COMMON_GUESS_DOC
+
+
+class SkewedVoigtModel(Model):
+    r"""A variation of the Skewed Gaussian, this applies the same skewing
+      to a Voigt distribution (see
+      https://en.wikipedia.org/wiki/Voigt_distribution).  It has Parameters
+      ``amplitude`` (:math:`A`), ``center`` (:math:`\mu`), ``sigma``
+      (:math:`\sigma`), and ``gamma`` (:math:`\gamma`), as usual for a
+      Voigt distribution, and add a Parameter ``skew``.  In addition,
+      parameters ``fwhm`` and ``height`` are included as constraints to
+      report full width at half maximum and maximum peak height, of the
+      Voigt distribution, respectively.
+
+    .. math::
+
+
+       f(x; A, \mu, \sigma, \gamma, \rm{skew}) = {\rm{Voigt}}(x; A, \mu, \sigma, \gamma)
+       \Bigl\{ 1 +  {\operatorname{erf}}\bigl[
+       \frac{{\rm{skew}}(x-\mu)}{\sigma\sqrt{2}}
+       \bigr] \Bigr\}
+
+    where :func:`erf` is the error function.
+
+    """
+
+    fwhm_factor = 3.60131
+    height_factor = 1./np.sqrt(2*np.pi)
+
+    def __init__(self, independent_vars=['x'], prefix='', nan_policy='raise',
+                 **kwargs):
+        kwargs.update({'prefix': prefix, 'nan_policy': nan_policy,
+                       'independent_vars': independent_vars})
+        super(SkewedVoigtModel, self).__init__(skewed_voigt, **kwargs)
+        self._set_paramhints_prefix()
+
+    def _set_paramhints_prefix(self):
+        self.set_param_hint('sigma', min=0)
+        self.set_param_hint('gamma', expr='%ssigma' % self.prefix)
+        self.set_param_hint('height', expr=height_expr(self))
+        self.set_param_hint('fwhm', expr=fwhm_expr(self))
+
+    def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -894,6 +962,7 @@ class DonaichModel(Model):
         self.set_param_hint('height', expr=fmt.format(prefix=self.prefix))
 
     def guess(self, data, x=None, negative=False, **kwargs):
+        """Estimate initial model parameter values from data."""
         pars = guess_from_peak(self, data, x, negative, ampscale=0.5)
         return update_param_vals(pars, self.prefix, **kwargs)
 
@@ -918,6 +987,7 @@ class PowerLawModel(Model):
         super(PowerLawModel, self).__init__(powerlaw, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         try:
             expon, amp = np.polyfit(np.log(x+1.e-14), np.log(data+1.e-14), 1)
         except TypeError:
@@ -948,6 +1018,7 @@ class ExponentialModel(Model):
         super(ExponentialModel, self).__init__(exponential, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         try:
             sval, oval = np.polyfit(x, np.log(abs(data)+1.e-15), 1)
         except TypeError:
@@ -995,6 +1066,7 @@ class StepModel(Model):
         super(StepModel, self).__init__(step, **kwargs)
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         if x is None:
             return
         ymin, ymax = min(data), max(data)
@@ -1060,6 +1132,7 @@ class RectangleModel(Model):
                                                                 self.prefix))
 
     def guess(self, data, x=None, **kwargs):
+        """Estimate initial model parameter values from data."""
         if x is None:
             return
         ymin, ymax = min(data), max(data)
@@ -1121,7 +1194,7 @@ class ExpressionModel(Model):
 
         """
         # create ast evaluator, load custom functions
-        self.asteval = Interpreter()
+        self.asteval = Interpreter(max_time=3600.0)
         for name in lineshapes.functions:
             self.asteval.symtable[name] = getattr(lineshapes, name, None)
         if init_script is not None:
@@ -1165,6 +1238,7 @@ class ExpressionModel(Model):
         def _eval(**kwargs):
             for name, val in kwargs.items():
                 self.asteval.symtable[name] = val
+            self.asteval.start_time = time.time()
             return self.asteval.run(self.astcode)
 
         kws["nan_policy"] = nan_policy
