@@ -6,10 +6,12 @@ from copy import deepcopy
 import json
 
 from asteval import Interpreter, get_ast_names, valid_symbol_name
-from numpy import arcsin, array, cos, inf, nan, sin, sqrt
+from numpy import arcsin, array, cos, inf, isclose, nan, sin, sqrt
 import scipy.special
 import uncertainties
 
+from .jsonutils import decode4js, encode4js
+
 SCIPY_FUNCTIONS = {'gamfcn': scipy.special.gamma}
 for name in ('erf', 'erfc', 'wofz'):
     SCIPY_FUNCTIONS[name] = getattr(scipy.special, name)
@@ -21,31 +23,6 @@ def check_ast_errors(expr_eval):
         expr_eval.raise_exception(None)
 
 
-def isclose(x, y, rtol=1e-5, atol=1e-8):
-    """Check whether two numbers are the same within a tolerance.
-
-    abs(`x` - `y`) <= (`atol` + `rtol` * abs(`y`))
-
-    Parameters
-    ----------
-    x, y : float
-        Input values.
-    rtol : float, optional
-        The relative tolerance parameter.
-    atol : float, optional
-        The absolute tolerance parameter.
-
-    Returns
-    -------
-    bool
-        True if `x` and `y` are the same within tolerance, otherwise False.
-
-    """
-    if x in (-inf, inf, nan) or y in (-inf, inf, nan):
-        return (x == y)
-    return (abs(x - y) <= atol + rtol * abs(y))
-
-
 class Parameters(OrderedDict):
     """An ordered dictionary of all the Parameter objects required to
     specify a fit model. All minimization and Model fitting routines in
@@ -66,13 +43,16 @@ class Parameters(OrderedDict):
 
     """
 
-    def __init__(self, asteval=None, *args, **kwds):
+    def __init__(self, asteval=None, usersyms=None, *args, **kwds):
         """
         Arguments
         ---------
         asteval : :class:`asteval.Interpreter`, optional
             Instance of the asteval Interpreter to use for constraint
             expressions. If None, a new interpreter will be created.
+            Warning: *deprecated, use usersyms if possible*
+        usersyms : dictionary of symbols to add to the
+            :class:`asteval.Interpreter`.
         *args : optional
             Arguments.
         **kwds : optional
@@ -80,10 +60,18 @@ class Parameters(OrderedDict):
 
         """
         super(Parameters, self).__init__(self)
-        self._asteval = asteval
 
-        if asteval is None:
-            self._asteval = Interpreter(usersyms=SCIPY_FUNCTIONS)
+        self._asteval = asteval
+        if self._asteval is None:
+            self._asteval = Interpreter()
+
+        _syms = {}
+        _syms.update(SCIPY_FUNCTIONS)
+        if usersyms is not None:
+            _syms.update(usersyms)
+        for key, val in _syms.items():
+            self._asteval.symtable[key] = val
+
         self.update(*args, **kwds)
 
     def copy(self):
@@ -95,8 +83,12 @@ class Parameters(OrderedDict):
         return self.__deepcopy__(None)
 
     def __deepcopy__(self, memo):
-        """Parameters.deepcopy() needs to make sure that asteval is available
-        and that all individual Parameter objects are copied."""
+        """Implementation of Parameters.deepcopy().
+
+        The method needs to make sure that asteval is available and that all
+        individual Parameter objects are copied.
+
+        """
         _pars = Parameters(asteval=None)
 
         # find the symbols that were added by users, not during construction
@@ -398,7 +390,7 @@ class Parameters(OrderedDict):
         """
         params = [p.__getstate__() for p in self.values()]
         sym_unique = self._asteval.user_defined_symbols()
-        unique_symbols = {key: deepcopy(self._asteval.symtable[key])
+        unique_symbols = {key: encode4js(deepcopy(self._asteval.symtable[key]))
                           for key in sym_unique}
         return json.dumps({'unique_symbols': unique_symbols,
                            'params': params}, **kws)
@@ -428,7 +420,7 @@ class Parameters(OrderedDict):
         """
         self.clear()
 
-        tmp = json.loads(s, **kws)
+        tmp = decode4js(json.loads(s, **kws))
         state = {'unique_symbols': tmp['unique_symbols'],
                  'params': []}
         for parstate in tmp['params']:
@@ -636,17 +628,15 @@ class Parameter(object):
             self.max = inf
         if self.min is None:
             self.min = -inf
-        if self._val is not None:
-            if self.min > self.max:
-                self.min, self.max = self.max, self.min
-            if isclose(self.min, self.max, atol=1e-13, rtol=1e-13):
-                raise ValueError("Parameter '%s' has min == max" % self.name)
-
-            if self._val > self.max:
-                self._val = self.max
-            if self._val < self.min:
-                self._val = self.min
-        elif self._expr is None:
+        if self._val is None:
+            self._val = -inf
+        if self.min > self.max:
+            self.min, self.max = self.max, self.min
+        if isclose(self.min, self.max, atol=1e-13, rtol=1e-13):
+            raise ValueError("Parameter '%s' has min == max" % self.name)
+        if self._val > self.max:
+            self._val = self.max
+        if self._val < self.min:
             self._val = self.min
         self.setup_bounds()
 
@@ -668,7 +658,7 @@ class Parameter(object):
         self._init_bounds()
 
     def __repr__(self):
-        """Returns printable representation of a Parameter object."""
+        """Return printable representation of a Parameter object."""
         s = []
         if self.name is not None:
             s.append("'%s'" % self.name)
@@ -828,6 +818,8 @@ class Parameter(object):
         if val is None:
             self._expr_ast = None
         if val is not None and self._expr_eval is not None:
+            self._expr_eval.error = []
+            self._expr_eval.error_msg = None
             self._expr_ast = self._expr_eval.parse(val)
             check_ast_errors(self._expr_eval)
             self._expr_deps = get_ast_names(self._expr_ast)
